{% extends "defindex.html" %}
{% block tables %}

    <p>
    Abjad helps composers build up complex pieces of music notation
    in an iterative and incremental way. 
    Use Abjad to create a symbolic representation of all the notes, 
    rests, staves, tuplets, beams and slurs in any score.
    Because Abjad extends the Python programming language,
    you can use Abjad to make systematic changes to your music as you work.
    And because Abjad wraps the powerful LilyPond music notation package,
    you can use Abjad to control the typographic details of the symbols on 
    the page.
    </p>

    <img src="_static/rhythmic_staves.png">

    <hr/>

    <p><strong>Documentation</strong></p>
    <table class="contentstable" align="center"><tr>
        <td width="50%">
            <p class="biglink">
                <a class="biglink" href="{{ pathto("start_here/index") }}">Start here</a><br/>
                <span class="linkdescr">introduction and installation</span></p>
            <p class="biglink">
                <a class="biglink" href="{{ pathto("system_overview/index") }}">System overview</a><br/>
                <span class="linkdescr">important concepts</span></p>
            <p class="biglink">
                <a class="biglink" href="{{ pathto("examples/index") }}">Examples</a><br/>
                <span class="linkdescr">from the literature</span></p>
            <p class="biglink">
                <a class="biglink" href="{{ pathto("tutorials/index") }}">Tutorials</a><br/>
                <span class="linkdescr">step-by-step instructions for beginners</span></p>
        </td><td width="50%">
            <p class="biglink">
                <a class="biglink" href="{{ pathto("reference_manual/index") }}">Reference manual</a><br/>
                <span class="linkdescr">working with Abjad</span></p>
            <p class="biglink">
                <a class="biglink" href="{{
                pathto("developer_documentation/index") }}">For developers</a><br/>
                <span class="linkdescr">guidelines for contributors</span></p>
            <p class="biglink">
                <a class="biglink" href="{{ pathto("appendices/index") }}">Appendices</a><br/>
                <span class="linkdescr">reference tables</span></p>
            <p class="biglink">
                <a class="biglink" href="{{ pathto("api/index") }}">Abjad API</a><br/>
                <span class="linkdescr">the complete reference</span></p>
        </td></tr>
    </table>

    <hr/>

    <p><strong>In conversation</strong></p>
    <table class="contentstable" align="center">
        <tr>
        <td width="50%">
            <p class="biglink">
                <a class="biglink" href="{{ pathto("in_conversation/from_trevor_josiah_and_victor/index") }}">From Trevor, Josiah & Víctor</a><br/>
                <span class="linkdescr">greetings from the Abjad developers</span></p>
        </td>
        </tr>
    </table>

    <hr/>

    <p><strong>Download</strong></p>

    <ul>
        <li>Install Abjad with <a href="http://www.pip-installer.org/en/latest/installing.html">pip</a>:
            <code>sudo pip install abjad</code>
        </li>
        <li>Download an archive from <a href="https://pypi.python.org/pypi">PyPi</a>, the Python Package Index:
            <a href="https://pypi.python.org/pypi/Abjad/2.11">pypi.python.org/pypi/Abjad</a></li>
        <li>Check-out a cutting-edge copy from our git repository:
            <a href="https://github.com/Abjad/abjad">https://github.com/Abjad/abjad</a></li>
    </ul>

    <hr/>

    <p><strong>Mailing lists</strong></p>

    <ul>
        <li><a href="http://groups.google.com/group/abjad-user">abjad-user</a></li>
        <li><a href="http://groups.google.com/group/abjad-devel">abjad-devel</a></li>
    </ul>

{% endblock %}
