# -*- encoding: utf-8 -*-
lilypond_version = "2.16.1"

contexts = {
    "ChoirStaff": {
        "accepts": set([
            "Staff",
            "StaffGroup",
            "ChoirStaff",
            "FiguredBass",
            "ChordNames",
            "Lyrics",
            "PianoStaff",
            "GrandStaff",
            "RhythmicStaff",
            "DrumStaff",
        ]),
        "aliases": set([]),
        "consists": set([
            "System_start_delimiter_engraver",
            "Instrument_name_engraver",
            "Vertical_align_engraver",
        ]),
        "grobs": set([
            "SystemStartSquare",
            "SystemStartBrace",
            "SystemStartBracket",
            "SystemStartBar",
            "InstrumentName",
            "VerticalAlignment",
        ]),
    },
    "ChordNames": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Hara_kiri_engraver",
            "Chord_name_engraver",
            "Separating_line_group_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "VerticalAxisGroup",
            "ChordName",
            "StaffSpacing",
        ]),
    },
    "CueVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Ligature_bracket_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "LigatureBracket",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "Devnull": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
            "Staff",
        ]),
        "consists": set([]),
        "grobs": set([]),
    },
    "DrumStaff": {
        "accepts": set([
            "DrumVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Time_signature_engraver",
            "Clef_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "TimeSignature",
            "Clef",
            "OctavateEight",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "DrumVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Drum_notes_engraver",
            "Grob_pq_engraver",
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Breathing_sign_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "NoteHead",
            "Script",
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "BreathingSign",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "Dynamics": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Tweak_engraver",
            "Axis_group_engraver",
            "Text_spanner_engraver",
            "Text_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Script_engraver",
            "Piano_pedal_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "VerticalAxisGroup",
            "TextSpanner",
            "TextScript",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "Script",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "BarLine",
        ]),
    },
    "FiguredBass": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Hara_kiri_engraver",
            "Separating_line_group_engraver",
            "Figured_bass_engraver",
        ]),
        "grobs": set([
            "VerticalAxisGroup",
            "StaffSpacing",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
        ]),
    },
    "FretBoards": {
        "accepts": set([]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Instrument_name_engraver",
            "Font_size_engraver",
            "Separating_line_group_engraver",
            "Hara_kiri_engraver",
            "Output_property_engraver",
            "Fretboard_engraver",
        ]),
        "grobs": set([
            "InstrumentName",
            "StaffSpacing",
            "VerticalAxisGroup",
            "FretBoard",
        ]),
    },
    "Global": {
        "accepts": set([
            "Score",
        ]),
        "aliases": set([]),
        "consists": set([]),
        "grobs": set([]),
    },
    "GrandStaff": {
        "accepts": set([
            "Staff",
            "ChordNames",
            "Dynamics",
            "FiguredBass",
            "Lyrics",
            "TabStaff",
            "DrumStaff",
            "RhythmicStaff",
        ]),
        "aliases": set([]),
        "consists": set([
            "Vertical_align_engraver",
            "System_start_delimiter_engraver",
            "Span_arpeggio_engraver",
            "Span_bar_stub_engraver",
            "Span_bar_engraver",
            "Instrument_name_engraver",
        ]),
        "grobs": set([
            "VerticalAlignment",
            "SystemStartSquare",
            "SystemStartBrace",
            "SystemStartBracket",
            "SystemStartBar",
            "Arpeggio",
            "SpanBarStub",
            "SpanBar",
            "InstrumentName",
        ]),
    },
    "GregorianTranscriptionStaff": {
        "accepts": set([
            "GregorianTranscriptionVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Accidental_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Time_signature_engraver",
            "Key_engraver",
            "Clef_engraver",
            "Ottava_spanner_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "TimeSignature",
            "KeyCancellation",
            "KeySignature",
            "Clef",
            "OctavateEight",
            "OttavaBracket",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "GregorianTranscriptionVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Episema_engraver",
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Ligature_bracket_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "Episema",
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "LigatureBracket",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "KievanStaff": {
        "accepts": set([
            "KievanVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Accidental_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Key_engraver",
            "Clef_engraver",
            "Ottava_spanner_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "KeyCancellation",
            "KeySignature",
            "Clef",
            "OctavateEight",
            "OttavaBracket",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "KievanVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Ligature_bracket_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "LigatureBracket",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "Lyrics": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Pure_from_neighbor_engraver",
            "Hara_kiri_engraver",
            "Font_size_engraver",
            "Instrument_name_engraver",
            "Stanza_number_engraver",
            "Tweak_engraver",
            "Hyphen_engraver",
            "Extender_engraver",
            "Lyric_engraver",
        ]),
        "grobs": set([
            "VerticalAxisGroup",
            "InstrumentName",
            "StanzaNumber",
            "LyricHyphen",
            "LyricSpace",
            "LyricExtender",
            "LyricText",
        ]),
    },
    "MensuralStaff": {
        "accepts": set([
            "MensuralVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Custos_engraver",
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Accidental_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Time_signature_engraver",
            "Key_engraver",
            "Clef_engraver",
            "Ottava_spanner_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "Custos",
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "TimeSignature",
            "KeyCancellation",
            "KeySignature",
            "Clef",
            "OctavateEight",
            "OttavaBracket",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "MensuralVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Mensural_ligature_engraver",
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "MensuralLigature",
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "NoteNames": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Separating_line_group_engraver",
            "Note_name_engraver",
            "Tie_engraver",
            "Axis_group_engraver",
        ]),
        "grobs": set([
            "StaffSpacing",
            "NoteName",
            "Tie",
            "TieColumn",
            "VerticalAxisGroup",
        ]),
    },
    "PetrucciStaff": {
        "accepts": set([
            "PetrucciVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Custos_engraver",
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Accidental_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Time_signature_engraver",
            "Key_engraver",
            "Clef_engraver",
            "Ottava_spanner_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "Custos",
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "TimeSignature",
            "KeyCancellation",
            "KeySignature",
            "Clef",
            "OctavateEight",
            "OttavaBracket",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "PetrucciVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Mensural_ligature_engraver",
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "MensuralLigature",
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "PianoStaff": {
        "accepts": set([
            "Staff",
            "ChordNames",
            "Dynamics",
            "FiguredBass",
            "Lyrics",
            "TabStaff",
            "DrumStaff",
            "RhythmicStaff",
        ]),
        "aliases": set([
            "GrandStaff",
        ]),
        "consists": set([
            "Keep_alive_together_engraver",
            "Vertical_align_engraver",
            "Vertical_align_engraver",
            "System_start_delimiter_engraver",
            "Span_arpeggio_engraver",
            "Span_bar_stub_engraver",
            "Span_bar_engraver",
            "Instrument_name_engraver",
        ]),
        "grobs": set([
            "VerticalAlignment",
            "VerticalAlignment",
            "SystemStartSquare",
            "SystemStartBrace",
            "SystemStartBracket",
            "SystemStartBar",
            "Arpeggio",
            "SpanBarStub",
            "SpanBar",
            "InstrumentName",
        ]),
    },
    "RhythmicStaff": {
        "accepts": set([
            "Voice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Ledger_line_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Time_signature_engraver",
            "Pitch_squash_engraver",
            "Staff_symbol_engraver",
            "Bar_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "LedgerLineSpanner",
            "VerticalAxisGroup",
            "InstrumentName",
            "TimeSignature",
            "StaffSymbol",
            "BarLine",
            "DotColumn",
            "StaffSpacing",
        ]),
    },
    "Score": {
        "accepts": set([
            "Staff",
            "FiguredBass",
            "NoteNames",
            "Devnull",
            "PianoStaff",
            "ChoirStaff",
            "GrandStaff",
            "ChordNames",
            "Lyrics",
            "KievanStaff",
            "DrumStaff",
            "StaffGroup",
            "PetrucciStaff",
            "MensuralStaff",
            "GregorianTranscriptionStaff",
            "VaticanaStaff",
            "TabStaff",
            "RhythmicStaff",
            "FretBoards",
        ]),
        "aliases": set([
            "Timing",
        ]),
        "consists": set([
            "Footnote_engraver",
            "Beam_collision_engraver",
            "Concurrent_hairpin_engraver",
            "Parenthesis_engraver",
            "Bar_number_engraver",
            "Stanza_number_align_engraver",
            "Vertical_align_engraver",
            "Grace_spacing_engraver",
            "Spacing_engraver",
            "Break_align_engraver",
            "Metronome_mark_engraver",
            "Volta_engraver",
            "Mark_engraver",
            "System_start_delimiter_engraver",
            "Output_property_engraver",
            "Default_bar_line_engraver",
            "Timing_translator",
            "Staff_collecting_engraver",
            "Repeat_acknowledge_engraver",
            "Paper_column_engraver",
        ]),
        "grobs": set([
            "FootnoteItem",
            "FootnoteSpanner",
            "ParenthesesItem",
            "BarNumber",
            "VerticalAlignment",
            "GraceSpacing",
            "SpacingSpanner",
            "BreakAlignment",
            "BreakAlignGroup",
            "LeftEdge",
            "MetronomeMark",
            "VoltaBracket",
            "VoltaBracketSpanner",
            "RehearsalMark",
            "SystemStartSquare",
            "SystemStartBrace",
            "SystemStartBracket",
            "SystemStartBar",
            "PaperColumn",
            "NonMusicalPaperColumn",
        ]),
    },
    "Staff": {
        "accepts": set([
            "Voice",
            "CueVoice",
        ]),
        "aliases": set([]),
        "consists": set([
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Accidental_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Time_signature_engraver",
            "Key_engraver",
            "Clef_engraver",
            "Ottava_spanner_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "TimeSignature",
            "KeyCancellation",
            "KeySignature",
            "Clef",
            "OctavateEight",
            "OttavaBracket",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "StaffGroup": {
        "accepts": set([
            "Staff",
            "StaffGroup",
            "ChoirStaff",
            "FiguredBass",
            "ChordNames",
            "Lyrics",
            "TabStaff",
            "PianoStaff",
            "GrandStaff",
            "DrumStaff",
            "RhythmicStaff",
        ]),
        "aliases": set([]),
        "consists": set([
            "System_start_delimiter_engraver",
            "Output_property_engraver",
            "Span_arpeggio_engraver",
            "Span_bar_stub_engraver",
            "Span_bar_engraver",
            "Instrument_name_engraver",
            "Vertical_align_engraver",
        ]),
        "grobs": set([
            "SystemStartSquare",
            "SystemStartBrace",
            "SystemStartBracket",
            "SystemStartBar",
            "Arpeggio",
            "SpanBarStub",
            "SpanBar",
            "InstrumentName",
            "VerticalAlignment",
        ]),
    },
    "TabStaff": {
        "accepts": set([
            "TabVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Tab_staff_symbol_engraver",
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Time_signature_engraver",
            "Clef_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "StaffSymbol",
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "TimeSignature",
            "Clef",
            "OctavateEight",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "TabVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Tab_tie_follow_engraver",
            "Tab_note_heads_engraver",
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Breathing_sign_engraver",
            "Ligature_bracket_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "TabNoteHead",
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "BreathingSign",
            "LigatureBracket",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
        ]),
    },
    "VaticanaStaff": {
        "accepts": set([
            "VaticanaVoice",
            "CueVoice",
        ]),
        "aliases": set([
            "Staff",
        ]),
        "consists": set([
            "Custos_engraver",
            "Footnote_engraver",
            "Cue_clef_engraver",
            "Script_row_engraver",
            "Figured_bass_position_engraver",
            "Figured_bass_engraver",
            "Axis_group_engraver",
            "Instrument_name_engraver",
            "Piano_pedal_align_engraver",
            "Piano_pedal_engraver",
            "Accidental_engraver",
            "Rest_collision_engraver",
            "Grob_pq_engraver",
            "Collision_engraver",
            "Staff_symbol_engraver",
            "Ledger_line_engraver",
            "Key_engraver",
            "Clef_engraver",
            "Ottava_spanner_engraver",
            "Staff_collecting_engraver",
            "Dot_column_engraver",
            "Separating_line_group_engraver",
            "Font_size_engraver",
            "Pure_from_neighbor_engraver",
            "Bar_engraver",
            "Output_property_engraver",
        ]),
        "grobs": set([
            "Custos",
            "FootnoteItem",
            "FootnoteSpanner",
            "CueClef",
            "CueEndClef",
            "OctavateEight",
            "ScriptRow",
            "BassFigureAlignmentPositioning",
            "BassFigure",
            "BassFigureAlignment",
            "BassFigureBracket",
            "BassFigureContinuation",
            "BassFigureLine",
            "VerticalAxisGroup",
            "InstrumentName",
            "SostenutoPedalLineSpanner",
            "SustainPedalLineSpanner",
            "UnaCordaPedalLineSpanner",
            "PianoPedalBracket",
            "SostenutoPedal",
            "SustainPedal",
            "UnaCordaPedal",
            "Accidental",
            "AccidentalCautionary",
            "AccidentalPlacement",
            "AccidentalSuggestion",
            "RestCollision",
            "NoteCollision",
            "StaffSymbol",
            "LedgerLineSpanner",
            "KeyCancellation",
            "KeySignature",
            "Clef",
            "OctavateEight",
            "OttavaBracket",
            "DotColumn",
            "StaffSpacing",
            "BarLine",
        ]),
    },
    "VaticanaVoice": {
        "accepts": set([]),
        "aliases": set([
            "Voice",
        ]),
        "consists": set([
            "Episema_engraver",
            "Vaticana_ligature_engraver",
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "Episema",
            "VaticanaLigature",
            "DotColumn",
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
    "Voice": {
        "accepts": set([]),
        "aliases": set([]),
        "consists": set([
            "Instrument_switch_engraver",
            "Grace_engraver",
            "Tuplet_engraver",
            "Tie_engraver",
            "Slur_engraver",
            "Cluster_spanner_engraver",
            "Phrasing_slur_engraver",
            "Spanner_break_forbid_engraver",
            "Note_spacing_engraver",
            "Rhythmic_column_engraver",
            "Script_column_engraver",
            "Script_engraver",
            "Bend_engraver",
            "Fingering_engraver",
            "Dynamic_align_engraver",
            "New_dynamic_engraver",
            "Text_engraver",
            "Part_combine_engraver",
            "Slash_repeat_engraver",
            "Percent_repeat_engraver",
            "Double_percent_repeat_engraver",
            "Chord_tremolo_engraver",
            "New_fingering_engraver",
            "Auto_beam_engraver",
            "Grace_beam_engraver",
            "Beam_engraver",
            "Stem_engraver",
            "Tweak_engraver",
            "Rest_engraver",
            "Dots_engraver",
            "Note_heads_engraver",
            "Breathing_sign_engraver",
            "Ligature_bracket_engraver",
            "Glissando_engraver",
            "Note_head_line_engraver",
            "Repeat_tie_engraver",
            "Laissez_vibrer_engraver",
            "Forbid_line_break_engraver",
            "Grob_pq_engraver",
            "Trill_spanner_engraver",
            "Text_spanner_engraver",
            "Multi_measure_rest_engraver",
            "Arpeggio_engraver",
            "Output_property_engraver",
            "Pitched_trill_engraver",
            "Font_size_engraver",
        ]),
        "grobs": set([
            "InstrumentSwitch",
            "TupletBracket",
            "TupletNumber",
            "Tie",
            "TieColumn",
            "Slur",
            "ClusterSpanner",
            "ClusterSpannerBeacon",
            "PhrasingSlur",
            "NoteSpacing",
            "NoteColumn",
            "ScriptColumn",
            "Script",
            "BendAfter",
            "Fingering",
            "DynamicLineSpanner",
            "DynamicTextSpanner",
            "DynamicText",
            "Hairpin",
            "TextScript",
            "CombineTextScript",
            "DoubleRepeatSlash",
            "RepeatSlash",
            "PercentRepeat",
            "PercentRepeatCounter",
            "DoublePercentRepeat",
            "DoublePercentRepeatCounter",
            "Beam",
            "Fingering",
            "StringNumber",
            "StrokeFinger",
            "Script",
            "Beam",
            "Beam",
            "Beam",
            "Stem",
            "StemTremolo",
            "Rest",
            "Dots",
            "NoteHead",
            "BreathingSign",
            "LigatureBracket",
            "Glissando",
            "Glissando",
            "VoiceFollower",
            "RepeatTie",
            "RepeatTieColumn",
            "LaissezVibrerTie",
            "LaissezVibrerTieColumn",
            "TrillSpanner",
            "TextSpanner",
            "MultimeasureRest",
            "MultiMeasureRestNumber",
            "MultiMeasureRestText",
            "Arpeggio",
            "TrillPitchHead",
            "TrillPitchAccidental",
            "TrillPitchGroup",
        ]),
    },
}
